import * as fs from 'node:fs';
import * as path from 'node:path';
import { homedir } from 'node:os';
import * as readline from 'node:readline/promises';
import { stdin as input, stdout as output } from 'process';
import { Log } from '../logger.js';
export class ConfigService {
    fs = {
        write: (path, data) => fs.writeFileSync(path, JSON.stringify(data, null, 2)),
        read: (path) => fs.readFileSync(path, 'utf-8'),
        exists: (path) => fs.existsSync(path)
    };
    rl = {
        question: (rli, question) => rli.question(question)
    };
    fileName = '.alia.json';
    defaultConfig = {
        version: 1,
        meta: {
            gist: {
                token: "",
                id: ""
            }
        },
        options: {
            separator: "@"
        },
        alias: {
            test: {
                options: {
                    shell: false
                },
                command: [
                    "echo",
                    "alia",
                    "is",
                    "working!"
                ]
            }
        }
    };
    filePath = path.join(homedir(), this.fileName);
    _config;
    get config() {
        if (this._config) {
            return this._config;
        }
        const config = this.fs.read(this.filePath);
        try {
            this._config = JSON.parse(config);
            return this._config;
        }
        catch (e) {
            Log.info('Failed to load config:', this.filePath);
            Log.info('Make sure to run: al --init');
            throw e;
        }
    }
    getAlias(key) {
        return this.config.alias[key];
    }
    setAlias(key, command) {
        this.config.alias[key] = command;
        this.save(this.config);
    }
    removeAlias(key) {
        const { [key]: _, ...rest } = this.config.alias;
        this.config.alias = rest;
        this.save(this.config);
    }
    getSeparator() {
        return this.config.options.separator;
    }
    setSeparator(separator) {
        this.config.options.separator = separator
            ? separator
            : this.defaultConfig.options.separator;
        this.save(this.config);
    }
    getToken() {
        return this.config.meta.gist.token;
    }
    setToken(token) {
        this.config.meta.gist.token = token;
        this.save(this.config);
    }
    getGistId() {
        return this.config.meta.gist.id;
    }
    setGistId(id) {
        this.config.meta.gist.id = id;
        this.save(this.config);
    }
    getMetaData() {
        return this.config.meta;
    }
    async init() {
        if (this.fs.exists(this.filePath)) {
            const answer = await this.rl.question(readline.createInterface({ input, output }), `Config already exists at: ${this.filePath}\nWould you like to overwrite (y/n): `);
            if (answer.toLowerCase() !== 'y') {
                return;
            }
        }
        this.save(this.defaultConfig);
        Log.info('Created config:', this.filePath);
    }
    save(config) {
        this.fs.write(this.filePath, config);
    }
}
