import pkg from '../../package.json' assert { type: 'json' };
import { Log } from '../logger.js';
export class OptionService {
    configService;
    gistService;
    constructor(configService, gistService) {
        this.configService = configService;
        this.gistService = gistService;
    }
    flags = [
        {
            full: 'version',
            short: 'v',
            action: () => Promise.resolve(this.version())
        },
        {
            full: 'help',
            short: 'h',
            action: () => Promise.resolve(this.help())
        },
        {
            full: 'set',
            short: 's',
            modifiers: ['shell'],
            action: (params) => Promise.resolve(this.set(params))
        },
        {
            full: 'remove',
            short: 'r',
            action: (params) => Promise.resolve(this.remove(params))
        },
        {
            full: 'list',
            short: 'l',
            modifiers: ['sort'],
            action: (params) => Promise.resolve(this.list(params))
        },
        {
            full: 'conf',
            short: 'c',
            modifiers: ['separator', 'token', 'gist', 'path'],
            action: (params) => Promise.resolve(this.conf(params))
        },
        {
            full: 'sync',
            short: 'sy',
            modifiers: ['push', 'pull'],
            action: (params) => this.sync(params)
        }
    ];
    version() {
        Log.info(pkg.version);
    }
    help() {
        const separator = this.configService.getSeparator();
        Log.info(`
      Usage

        $ al [options] [alias] [${separator}] [command]

      Options

        --version, -v     show version
        --help, -h        show this

        --set, -s         set alias
          --shell             enable shell mode

        --remove, -r      remove alias

        --list, -l        list available alias
          --sort              sort alphabetically

        --conf, -c        change alia configuration
          --separator=[string]                set alias separator (default: @)
          --=token=<your github api token>    set the api token for gist sync
          --gist=<your gist id>               set the gist id to use for sync
          --path                              show config file path

        --sync, -sy        backup/restore your config (default: restore)
          --push             backup your current config
          --pull             restore config from gist

      Examples

        $ al -s gp ${separator} git push
          > Added: gp ${separator} git push

        $ al gp
          > git push

        $ al -r gp
          > Removed: gp
  `);
    }
    set({ args, modifiers }) {
        const separator = this.configService.getSeparator();
        const separatorIndex = args.indexOf(separator);
        if (separatorIndex === -1) {
            throw new Error(`Invalid Input, missing separator: '${separator}'`);
        }
        const key = args[separatorIndex - 1];
        let command = args.slice(separatorIndex + 1);
        if (!key || command.length === 0) {
            throw new Error('Invalid arguments passed');
        }
        if (command.length === 1) {
            command = command[0].split(' ');
        }
        const alias = this.configService.getAlias(key);
        if (alias) {
            Log.info(`Unset alias: ${key} ${separator} ${alias.command.join(' ')}`);
        }
        this.configService.setAlias(key, {
            options: {
                shell: !!modifiers.shell
            },
            command
        });
        Log.info(`Set alias: ${key} ${separator} ${command.join(' ')}`);
    }
    remove({ args: [key] }) {
        if (!this.configService.getAlias(key)) {
            throw new Error(`Alias '${key}' does not exist`);
        }
        this.configService.removeAlias(key);
        Log.info(`Removed alias: ${key}`);
    }
    mapList(alias) {
        return Object.keys(alias)
            .map(key => `${key} \t${this.configService.getSeparator()} \t${alias[key].command.join(' ')}`);
    }
    list({ modifiers }) {
        const l = this.mapList(this.configService.config.alias);
        Log.info((modifiers.sort ? l.sort() : l).join('\n'));
    }
    async sync({ modifiers }) {
        if (Object.keys(modifiers).length === 0 || modifiers.pull) {
            return await this.gistService.pull();
        }
        if (modifiers.push) {
            return await this.gistService.push();
        }
        throw new Error('Invalid arguments passed');
    }
    conf({ modifiers, data }) {
        if (Object.keys(modifiers).length === 0) {
            throw new Error('Invalid arguments passed');
        }
        if (modifiers.separator) {
            this.configService.setSeparator(data.separator);
            Log.info(`Set the separator to:`, this.configService.getSeparator());
        }
        if (modifiers.gist) {
            if (!data.gist) {
                throw new Error('No gist id provided');
            }
            this.configService.setGistId(data.gist);
        }
        if (modifiers.token) {
            if (!data.token) {
                throw new Error('No token provided');
            }
            this.configService.setToken(data.token);
        }
        if (modifiers.path) {
            Log.info('Config path:', this.configService.filePath);
        }
    }
}
